@@ -1,6 +1,6 @@
 /**
  *	This file is part of devilspie2
- *	Copyright (C) 2005 Ross Burton, 2011-2013 Andreas Rönnquist
+ *	Copyright (C) 2005 Ross Burton, 2011-2017 Andreas Rönnquist
  *
  *	devilspie2 is free software: you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License as published
@@ -43,6 +43,10 @@
 #include "config.h"
 
 
+#if (GTK_MAJOR_VERSION >= 3)
+#define HAVE_GTK3
+#endif
+
 /**
  *
  */
@@ -52,11 +56,11 @@ static gboolean debug = FALSE;
 static gboolean emulate = FALSE;
 static gboolean show_version = FALSE;
 
-#ifdef HAVE_GTK3
-	// libwnck Version Information is only availible if you have
-	// libwnck 3.0 or later
-	static gboolean show_wnck_version = FALSE;
-#endif
+// libwnck Version Information is only availible if you have
+// libwnck 3.0 or later
+static gboolean show_wnck_version = FALSE;
+
+static gboolean show_lua_version = FALSE;
 
 static gchar *script_folder = NULL;
 static gchar *temp_folder = NULL;
@@ -65,24 +69,26 @@ GFileMonitor *mon = NULL;
 
 gchar *config_filename = NULL;
 
+WnckHandle *my_wnck_handle = NULL;
+
 /**
  *
  */
-static void load_list_of_scripts(WnckScreen *screen, WnckWindow *window,
-											GSList *file_list)
+static void load_list_of_scripts(WnckScreen *screen G_GNUC_UNUSED, WnckWindow *window,
+                                 GSList *file_list)
 {
 	GSList *temp_file_list = file_list;
 	// set the window to work on
 	set_current_window(window);
 
 	// for every file in the folder - load the script
-	if (file_window_open_list != NULL) {
+	if (event_lists[W_OPEN] != NULL) {
 
 		while(temp_file_list) {
 			gchar *filename = (gchar*)temp_file_list->data;
 
-			// is it a LUA file?
-			if (g_str_has_suffix((gchar*)filename,".lua")) {
+			// is it a Lua file?
+			if (g_str_has_suffix((gchar*)filename, ".lua")) {
 
 				// init the script, run it
 				if (!load_script(global_lua_state, filename)) {
@@ -99,12 +105,38 @@ static void load_list_of_scripts(WnckScreen *screen, WnckWindow *window,
 }
 
 
+static void window_name_changed_cb(WnckWindow *window)
+{
+	WnckScreen * screen = wnck_window_get_screen(window);
+	if(screen == NULL) return;
+
+	// Handle duplicate name-change events
+	// Simple method: just track the most recent event regardless of window
+	static WnckWindow *previous = NULL;
+	static char *prevname = NULL;
+
+	const char *newname = wnck_window_get_name(window);
+	if (window == previous && prevname && !strcmp (prevname, newname))
+		return;
+	// Store the info for the next event
+	free(prevname);
+	prevname = strdup(newname);
+	previous = window;
+
+	load_list_of_scripts(screen, window, event_lists[W_NAME_CHANGED]);
+}
+
 /**
  *
  */
 static void window_opened_cb(WnckScreen *screen, WnckWindow *window)
 {
-	load_list_of_scripts(screen, window, file_window_open_list);
+	load_list_of_scripts(screen, window, event_lists[W_OPEN]);
+	/*
+	Attach a listener to each window for window-specific changes
+	Safe to do this way as long as the 'user data' parameter is NULL
+	*/
+	g_signal_connect(window, "name-changed", (GCallback)window_name_changed_cb, NULL);
 }
 
 
@@ -113,7 +145,20 @@ static void window_opened_cb(WnckScreen *screen, WnckWindow *window)
  */
 static void window_closed_cb(WnckScreen *screen, WnckWindow *window)
 {
-	load_list_of_scripts(screen, window, file_window_close_list);
+	load_list_of_scripts(screen, window, event_lists[W_CLOSE]);
+}
+
+
+/**
+ *
+ */
+static void window_changed_cb(WnckScreen *screen, WnckWindow *window)
+{
+	WnckWindow *cur;
+
+	load_list_of_scripts(screen, window, event_lists[W_BLUR]);
+	cur = wnck_screen_get_active_window(screen);
+	load_list_of_scripts(screen, cur, event_lists[W_FOCUS]);
 }
 
 
@@ -132,12 +177,14 @@ void init_screens()
 #endif
 
 	for (i=0; i<num_screens; i++) {
-		WnckScreen *screen = wnck_screen_get(i);
+		WnckScreen *screen = wnck_handle_get_screen(my_wnck_handle, i);
 
 		g_signal_connect(screen, "window-opened",
-								(GCallback)window_opened_cb, NULL);
+		                 (GCallback)window_opened_cb, NULL);
 		g_signal_connect(screen, "window-closed",
-								(GCallback)window_closed_cb, NULL);
+		                 (GCallback)window_closed_cb, NULL);
+		g_signal_connect(screen, "active-window-changed",
+		                 (GCallback)window_changed_cb, NULL);
 	}
 }
 
@@ -148,12 +195,10 @@ void init_screens()
 void devilspie_exit()
 {
 	clear_file_lists();
-
-	if (temp_folder != NULL) g_free(temp_folder);
-
-	if (mon) g_object_unref(mon);
-
-	if (config_filename) g_free(config_filename);
+	g_free(temp_folder);
+	if (mon)
+		g_object_unref(mon);
+	g_free(config_filename);
 }
 
 
@@ -162,16 +207,13 @@ void devilspie_exit()
  */
 static void signal_handler(int sig)
 {
-	printf("\n");
-	printf("%s", _("Received signal:"));
-	printf(" %d (%s)\n", sig, strsignal(sig));
+	printf("\n%s %d (%s)\n", _("Received signal:"), sig, strsignal(sig));
 
 	done_script_error_messages();
 
 	if (sig == SIGINT) {
 		exit(EXIT_FAILURE);
 	}
-
 }
 
 
@@ -203,51 +245,51 @@ void print_list(GSList *list)
  */
 void print_script_lists()
 {
+	gboolean have_any_files = FALSE;
+	win_event_type i;
+
 	if (debug)
 		printf("------------\n");
 
-	if ((file_window_open_list == NULL) && (file_window_close_list == NULL)) {
-		printf("%s", _("No script files found in the script folder - exiting."));
-		printf("\n\n");
+	for (i = 0; i < W_NUM_EVENTS; i++) {
+		if (event_lists[i])
+			have_any_files = TRUE;
+		// If we are running debug mode - print the list of files:
+		if (debug) {
+			printf(_("List of Lua files handling \"%s\" events in folder:"),
+				   event_names[i]);
+			printf("\n");
+			if (event_lists[i]) {
+				print_list(event_lists[i]);
+			}
+		}
+	}
+
+	if (!have_any_files) {
+		printf("%s\n\n", _("No script files found in the script folder - exiting."));
 		exit(EXIT_SUCCESS);
 	}
-
-	// If we are running debug mode - print the list of files:
-	if (debug) {
-		printf(_("List of LUA files handling \"%s\" events in folder:"),
-			"window_open");
-		printf("\n");
-		if (file_window_open_list)
-			print_list(file_window_open_list);
-	}
-
-	if (debug) {
-		printf(_("List of LUA files handling \"%s\" events in folder:"),
-			"window_close");
-		printf("\n");
-		if (file_window_close_list)
-			print_list(file_window_close_list);
-	}
 }
 
 
 /**
  *
  */
-void folder_changed_callback(GFileMonitor *mon,
-										GFile *first_file,
-										GFile *second_file,
-										GFileMonitorEvent event,
-										gpointer user_data)
+void folder_changed_callback(GFileMonitor *mon G_GNUC_UNUSED,
+                             GFile *first_file,
+                             GFile *second_file G_GNUC_UNUSED,
+                             GFileMonitorEvent event,
+                             gpointer user_data)
 {
 	gchar *our_filename = (gchar*)(user_data);
 
 	// If a file is created or deleted, we need to check the file lists again
 	if ((event == G_FILE_MONITOR_EVENT_CREATED) ||
-		 (event == G_FILE_MONITOR_EVENT_DELETED)) {
+	    (event == G_FILE_MONITOR_EVENT_DELETED)) {
 
 		clear_file_lists();
 
+		set_current_window(NULL);
 		load_config(our_filename);
 
 		if (debug)
@@ -269,6 +311,7 @@ void folder_changed_callback(GFileMonitor *mon,
 
 				clear_file_lists();
 
+				set_current_window(NULL);
 				load_config(our_filename);
 
 				print_script_lists();
@@ -286,21 +329,27 @@ void folder_changed_callback(GFileMonitor *mon,
  */
 int main(int argc, char *argv[])
 {
-	static const GOptionEntry options[]={
-		{ "debug",			'd',	0,	G_OPTION_ARG_NONE,		&debug,
-			N_("Print debug info to stdout")},
-		{ "emulate",		'e',	0,	G_OPTION_ARG_NONE,		&emulate,
-			N_("Don't apply any rules, only emulate execution")},
-		{ "folder",			'f',	0,	G_OPTION_ARG_STRING,		&script_folder,
-			N_("Search for scripts in this folder"),N_("FOLDER")},
-		{ "version",		'v',	0,	G_OPTION_ARG_NONE,		&show_version,
-			N_("Show Devilspie2 version and quit")},
-#ifdef HAVE_GTK3
+	static const GOptionEntry options[]= {
+		{ "debug",        'd', 0, G_OPTION_ARG_NONE,   &debug,
+		  N_("Print debug info to stdout"), NULL
+		},
+		{ "emulate",      'e', 0, G_OPTION_ARG_NONE,   &emulate,
+		  N_("Don't apply any rules, only emulate execution"), NULL
+		},
+		{ "folder",       'f', 0, G_OPTION_ARG_STRING, &script_folder,
+		  N_("Search for scripts in this folder"), N_("FOLDER")
+		},
+		{ "version",      'v', 0, G_OPTION_ARG_NONE,   &show_version,
+		  N_("Show Devilspie2 version and quit"), NULL
+		},
 		// libwnck Version Information is only availible if you have
 		// libwnck 3.0 or later
-		{ "wnck-version",	'w',	0,	G_OPTION_ARG_NONE,		&show_wnck_version,
-			N_("Show libwnck version and quit")},
-#endif
+		{ "wnck-version", 'w', 0, G_OPTION_ARG_NONE,   &show_wnck_version,
+		  N_("Show libwnck version and quit"), NULL
+		},
+		{ "lua-version",  'l', 0, G_OPTION_ARG_NONE,   &show_lua_version,
+		  N_("Show Lua version and quit"), NULL
+		},
 		{ NULL }
 	};
 
@@ -314,21 +363,20 @@ int main(int argc, char *argv[])
 	bind_textdomain_codeset(PACKAGE, "");
 	textdomain(PACKAGE);
 
-	gdk_init(&argc, &argv);
-
-	gchar *devilspie2_description =
-		g_strdup_printf(_("apply rules on windows"));
+	gchar *devilspie2_description = g_strdup_printf(_("apply rules on windows"));
 
 	gchar *full_desc_string = g_strdup_printf("- %s", devilspie2_description);
 
-	context=g_option_context_new(full_desc_string);
-	g_option_context_add_main_entries(context,options,NULL);
+	context = g_option_context_new(full_desc_string);
+	g_option_context_add_main_entries(context, options, NULL);
 	if (!g_option_context_parse(context, &argc, &argv, &error)) {
 		g_print(_("option parsing failed: %s"), error->message);
 		printf("\n");
 		exit(EXIT_FAILURE);
 	}
 
+	gdk_init(&argc, &argv);
+
 	g_free(full_desc_string);
 	g_free(devilspie2_description);
 
@@ -336,17 +384,16 @@ int main(int argc, char *argv[])
 	if (script_folder == NULL) {
 
 		temp_folder = g_build_path(G_DIR_SEPARATOR_S,
-										g_get_user_config_dir(),
-										"devilspie2",
-										NULL);
+		                           g_get_user_config_dir(),
+		                           "devilspie2",
+		                           NULL);
 
 		// check if the folder does exist
 		if (!g_file_test(temp_folder, G_FILE_TEST_IS_DIR)) {
 
 			// - and if it doesn't, create it.
 			if (g_mkdir(temp_folder, 0700) != 0) {
-				printf("%s", _("Couldn't create the default folder for devilspie2 scripts."));
-				printf("\n");
+				printf("%s\n", _("Couldn't create the default folder for devilspie2 scripts."));
 				exit(EXIT_FAILURE);
 			}
 		}
@@ -354,41 +401,58 @@ int main(int argc, char *argv[])
 		script_folder = temp_folder;
 	}
 
+	gboolean shown = FALSE;
 	if (show_version) {
-		printf("Devilspie2 v%s\n\n", DEVILSPIE2_VERSION);
-		exit(EXIT_SUCCESS);
+		printf("Devilspie2 v%s\n", DEVILSPIE2_VERSION);
+		shown = TRUE;
 	}
-#ifdef HAVE_GTK3
 	// libwnck Version Information is only availible if you have
 	// libwnck 3.0 or later
 	if (show_wnck_version) {
-		printf("libwnck v%d.%d.%d\n\n",
+#ifdef _DEBUG
+		printf("GTK v%d.%d.%d\n",
+		       GTK_MAJOR_VERSION,
+		       GTK_MINOR_VERSION,
+		       GTK_MICRO_VERSION);
+#endif
+#ifdef HAVE_GTK3
+		printf("libwnck v%d.%d.%d\n",
 		       WNCK_MAJOR_VERSION,
 		       WNCK_MINOR_VERSION,
 		       WNCK_MICRO_VERSION);
-		exit(EXIT_SUCCESS);
-	}
+#else
+		printf("libwnck v2.x\n");
 #endif
+		shown = TRUE;
+	}
+	if (show_lua_version) {
+		puts(LUA_VERSION);
+		shown = TRUE;
+	}
+	if (shown)
+		exit(0);
 
 #if (GTK_MAJOR_VERSION >= 3)
 	if (!GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
-		printf("%s", _("An X11 display is required for devilspie2."));
-		printf("\n\n");
+		printf("%s\n\n", _("An X11 display is required for devilspie2."));
 		return EXIT_FAILURE;
 	}
 
 #endif
 
 	if (init_script_error_messages()!=0) {
-		printf("%s", _("Couldn't init script error messages!"));
-		printf("\n");
+		printf("%s\n", _("Couldn't init script error messages!"));
 		exit(EXIT_FAILURE);
 	}
 
-	config_filename =
-		g_build_filename(script_folder, "devilspie2.lua", NULL);
+	// set the current window to NULL, we don't need to be able to modify
+	// the windows when reading the config
+	set_current_window(NULL);
 
-	if (load_config(config_filename)!=0) {
+	config_filename =
+	    g_build_filename(script_folder, "devilspie2.lua", NULL);
+
+	if (load_config(config_filename) != 0) {
 
 		devilspie_exit();
 		return EXIT_FAILURE;
@@ -397,13 +461,11 @@ int main(int argc, char *argv[])
 	if (debug) {
 
 		if (emulate) {
-			printf("%s", _("Running devilspie2 in debug and emulate mode."));
+			printf("%s\n\n", _("Running devilspie2 in debug and emulate mode."));
 		} else {
-			printf("%s", _("Running devilspie2 in debug mode."));
+			printf("%s\n\n", _("Running devilspie2 in debug mode."));
 		}
 
-		printf("\n\n");
-
 		printf(_("Using scripts from folder: %s"), script_folder);
 
 		printf("\n");
@@ -418,15 +480,14 @@ int main(int argc, char *argv[])
 	directory_file = g_file_new_for_path(script_folder);
 //	mon = g_file_monitor_directory(directory_file, G_FILE_MONITOR_WATCH_MOUNTS,
 	mon = g_file_monitor_directory(directory_file, G_FILE_MONITOR_NONE,
-												NULL, NULL);
+	                               NULL, NULL);
 	if (!mon) {
-		printf("%s", _("Couldn't create directory monitor!"));
-		printf("\n");
+		printf("%s\n", _("Couldn't create directory monitor!"));
 		return EXIT_FAILURE;
 	}
 
 	g_signal_connect(mon, "changed", G_CALLBACK(folder_changed_callback),
-		(gpointer)(config_filename));
+	                 (gpointer)(config_filename));
 
 	global_lua_state = init_script();
 	print_script_lists();
@@ -446,6 +507,7 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 
+	my_wnck_handle = wnck_handle_new(WNCK_CLIENT_TYPE_PAGER);
 	init_screens();
 
 	loop=g_main_loop_new(NULL, TRUE);
