@@ -1,6 +1,7 @@
+
 /**
  *	This file is part of devilspie2
- *	Copyright (C) 2012 Andreas Rönnquist
+ *	Copyright (C) 2012-2017 Andreas Rönnquist
  *
  *	devilspie2 is free software: you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License as published
@@ -27,11 +28,16 @@ extern gchar *one_indata_expected_error;
 extern gchar *two_indata_expected_error;
 extern gchar *four_indata_expected_error;
 
+extern gchar *one_or_two_indata_expected_error;
+extern gchar *two_or_three_indata_expected_error;
+
 extern gchar *number_expected_as_indata_error;
 extern gchar *boolean_expected_as_indata_error;
 
 extern gchar *string_expected_as_indata_error;
 
+extern gchar *number_or_string_expected_as_indata_error;
+
 extern gchar *integer_greater_than_zero_expected_error;
 extern gchar *could_not_find_current_viewport_error;
 
